type window
and gc
and drawable
and pixmap
and bitmap
and font
and rgb =
  { red: int;
    green: int;
    blue: int }
and color
and colormap;;

module Style =
struct
type operation =
    GDK_COPY
  | GDK_INVERT
  | GDK_XOR

(* GC fill types.
 *  Solid:
 *  Tiled:
 *  Stippled:
 *  OpaqueStippled:
 *)

type fill =
    SOLID
  | TILED
  | STIPPLED
  | OPAQUE_STIPPLED

(* GC line styles
 *  Solid:
 *  OnOffDash:
 *  DoubleDash:
 *)

type line =
    LINE_SOLID
  | LINE_ON_OFF_DASH
  | LINE_DOUBLE_DASH

type cap =
    CAP_NOT_LAST
  | CAP_BUTT
  | CAP_ROUND
  | CAP_PROJECTING

(* GC join styles
 *  JoinMiter:
 *  JoinRound:
 *  JoinBevel:
 *)

type join =
    JOIN_MITER
  | JOIN_ROUND
  | JOIN_BEVEL

type filterreturn =
    FILTER_CONTINUE    (* Event not handled, continue processesing *)
  | FILTER_TRANSLATE   (* Translated event stored *)
  | FILTER_REMOVE       (* Terminate processing, removing event *)

type visibilitystate= 
    VISIBILITY_UNOBSCURED
  | VISIBILITY_PARTIAL
  | VISIBILITY_FULLY_OBSCURED
end;;

exception Not_available;;
let _ = Callback.register_exception "color not available" Not_available;;

module Event = struct
type mask =
    EXPOSURE_MASK
  | POINTER_MOTION_MASK
  | POINTER_MOTION_HINT_MASK
  | BUTTON_MOTION_MASK
  | BUTTON1_MOTION_MASK
  | BUTTON2_MOTION_MASK
  | BUTTON3_MOTION_MASK
  | BUTTON_PRESS_MASK
  | BUTTON_RELEASE_MASK
  | KEY_PRESS_MASK
  | KEY_RELEASE_MASK
  | ENTER_NOTIFY_MASK
  | LEAVE_NOTIFY_MASK
  | FOCUS_CHANGE_MASK
  | STRUCTURE_MASK
  | PROPERTY_CHANGE_MASK
  | VISIBILITY_NOTIFY_MASK
  | PROXIMITY_IN_MASK
  | PROXIMITY_OUT_MASK

  module Extract =
  struct
    external expose_x: Glib.Pointer.t->int
      = "mlgdk_expose_event_extract_x"
    external expose_y: Glib.Pointer.t->int
      = "mlgdk_expose_event_extract_y"
    external expose_w: Glib.Pointer.t->int
      = "mlgdk_expose_event_extract_width"
    external expose_h: Glib.Pointer.t->int
      = "mlgdk_expose_event_extract_height"
    external key_string: Glib.Pointer.t->string
      = "mlgdk_key_event_extract_string"
    external key_keyval: Glib.Pointer.t->int
      = "mlgdk_key_event_extract_keyval"
    external key_state: Glib.Pointer.t->int
      = "mlgdk_key_event_extract_state"
  end
end;;

external drawable_from_window : window->drawable = "%identity";;
external drawable_from_pixmap : pixmap->drawable = "%identity";;
external drawable_from_bitmap : bitmap->drawable = "%identity";;

(* GC *)
external gc_new : window -> gc = "gdk_gc_new"
external gc_destroy : gc -> unit = "mlgdk_gc_destroy"
external gc_set_clip_rectangle : gc->int->int->int->int->unit =
  "mlgdk_gc_set_clip_rectangle"
external gc_set_foreground : gc->color->unit = "mlgdk_gc_set_foreground"
external gc_set_background : gc->color->unit = "mlgdk_gc_set_background"
external gc_set_font: gc->font->unit = "mlgdk_gc_set_font"
external gc_set_function: gc->Style.operation->unit = "mlgdk_gc_set_function"
external gc_set_fill: gc->Style.fill->unit = "mlgdk_gc_set_fill"
external gc_set_tile: gc->pixmap->unit = "mlgdk_gc_set_tile"
external gc_set_stipple: gc->pixmap->unit = "mlgdk_gc_set_stipple"
external gc_set_line_attributes:
  gc->int->Style.line->Style.cap->Style.join->unit = 
  "mlgdk_gc_set_line_attributes"

(* Pixmap *)
external pixmap_new: window->int->int->int->pixmap = "mlgdk_pixmap_new"
external pixmap_create_from_xpm: window->(bitmap ref option)
  ->(color ref option)->string->pixmap = "mlgdk_pixmap_create_from_xpm"

(* Color *)
external color_alloc: colormap->rgb->color= "mlgdk_color_alloc";;

(* Fonts *)
external font_load: string->font = "mlgdk_font_load";;

(* Drawing *)
external draw_point: drawable->gc->int->int->unit = "mlgdk_draw_point";;
external draw_line: drawable->gc->int->int->int->int->unit =
  "mlgdk_draw_line" "mlgdk_draw_line_native";;
external draw_rectangle: drawable->gc->bool->int->int->int->int->unit =
  "mlgdk_draw_rectangle" "mlgdk_draw_rectangle_native";;
external draw_arc: drawable->gc->bool->int->int->int->int->int->int->unit =
  "mlgdk_draw_arc" "mlgdk_draw_arc_native";;
external draw_polygon: drawable->gc->bool->((int*int) array)->unit =
  "mlgdk_draw_polygon";;
external draw_string: drawable->font->gc->int->int->string->unit =
  "mlgdk_draw_string" "mlgdk_draw_string_native";;
external draw_pixmap: drawable->gc->pixmap->int->int->int->int->int->int->unit=
  "mlgdk_draw_pixmap" "mlgdk_draw_pixmap_native";;
external draw_points: drawable->gc->((int*int) array)->unit =
  "mlgdk_draw_points"
external draw_segments: drawable->gc->((int*int*int*int) array)->unit =
  "mlgdk_draw_points"
external draw_lines: drawable->gc->((int*int) array)->unit =
  "mlgdk_draw_points"
